package org.japura.task;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.japura.Application;
import org.japura.task.ui.TaskExecutionUI;

/**
 * <P>
 * Copyright (C) 2012 Carlos Eduardo Leite de Andrade
 * <P>
 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * <P>
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * <P>
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <A
 * HREF="www.gnu.org/licenses/">www.gnu.org/licenses/</A>
 * <P>
 * For more information, contact: <A HREF="www.japura.org">www.japura.org</A>
 * <P>
 * 
 * @author Carlos Eduardo Leite de Andrade
 */
public abstract class AbstractTaskManager implements TaskManager{

  private Map<String, TaskExecutionUI> taskExecutionUIs =
	  new HashMap<String, TaskExecutionUI>();
  private TaskExecutionUI taskExecutionUI;
  private TaskSessionFactory taskSessionFactory =
	  new DefaultTaskSessionFactory();;
  private List<TaskManagerListener> listeners =
	  new ArrayList<TaskManagerListener>();

  protected TaskExecutionUI getTaskExecutionUI(Task<?> task,
											   TaskManagerEvent event) {
	TaskExecutionUI taskExecutionUI = null;
	try {
	  if (TaskManagerEvent.SUBMITTED.equals(event)) {
		taskExecutionUI = task.getTaskExecutionUI();
		if (taskExecutionUI != null) {
		  taskExecutionUIs.put(task.getId(), taskExecutionUI);
		}
	  } else if (TaskManagerEvent.BEFORE_EXECUTE.equals(event)) {
		taskExecutionUI = taskExecutionUIs.get(task.getId());
	  } else if (TaskManagerEvent.AFTER_EXECUTE.equals(event)) {
		taskExecutionUI = taskExecutionUIs.remove(task.getId());
	  }
	} catch (Exception e) {
	  Application.getHandlerExceptionManager().handle(
		  new TaskManagerExeception(e));
	}

	if (taskExecutionUI == null) {
	  taskExecutionUI = getTaskExecutionUI();
	}
	return taskExecutionUI;
  }

  @Override
  public TaskExecutionUI getTaskExecutionUI() {
	return taskExecutionUI;
  }

  @Override
  public void setTaskExecutionUI(TaskExecutionUI taskExecutionUI) {
	this.taskExecutionUI = taskExecutionUI;
  }

  @Override
  public synchronized void addListener(TaskManagerListener listener) {
	listeners.add(listener);
  }

  @Override
  public synchronized void removeListener(TaskManagerListener listener) {
	listeners.remove(listener);
  }

  @Override
  public synchronized List<TaskManagerListener> getListeners() {
	return Collections.unmodifiableList(listeners);
  }

  @Override
  public void setTaskSessionFactory(TaskSessionFactory taskSessionFactory) {
	if (taskSessionFactory == null) {
	  taskSessionFactory = new DefaultTaskSessionFactory();
	}
	this.taskSessionFactory = taskSessionFactory;
  }

  @Override
  public TaskSessionFactory getTaskSessionFactory() {
	return taskSessionFactory;
  }

  protected void checkSession(Task<?> task, TaskSession session) {
	if (session == null) {
	  throw new TaskExeception("NULL TaskSession");
	}

	SessionRequired sr = task.getClass().getAnnotation(SessionRequired.class);
	if (sr != null && session.getClass().equals(sr.sessionClass()) == false) {
	  throw new TaskExeception("Task '" + task.getClass().getName()
		  + "' requires the session class '" + sr.sessionClass()
		  + "' but is using '" + session.getClass() + "'");
	}
  }

  @Override
  public void submit(TaskSequence taskSequence) {
	submit(taskSequence.getFirstTask(), null, false);
  }

  @Override
  public void submit(TaskSequence taskSequence, TaskSession session) {
	submit(taskSequence.getFirstTask(), session, false);
  }

  @Override
  public void submit(Task<?> task) {
	submit(task, null, false);
  }

  @Override
  public void submit(Task<?> task, TaskSession session) {
	submit(task, session, false);
  }

  @Override
  public void submit(Task<?> task, boolean isParallel) {
	submit(task, null, isParallel);
  }

  @Override
  public void fireTaskManagerListeners(TaskManagerEvent event, Task<?> task) {
	if (event == null) {
	  return;
	}

	try {
	  for (TaskManagerListener listener : listeners) {
		if (TaskManagerEvent.BEFORE_EXECUTE.equals(event)) {
		  listener.beforeExecute(task);
		} else if (TaskManagerEvent.AFTER_EXECUTE.equals(event)) {
		  listener.afterExecute(task);
		} else if (TaskManagerEvent.SUBMITTED.equals(event)) {
		  listener.submitted(task);
		}
	  }
	} catch (Exception e) {
	  Application.getHandlerExceptionManager().handle(
		  new TaskManagerExeception(e));
	}
  }

  @Override
  public void fireTaskExecutionUIs(TaskManagerEvent event, Task<?> task) {
	if (event == null) {
	  return;
	}

	TaskExecutionUI taskExecutionUI = getTaskExecutionUI(task, event);

	if (taskExecutionUI != null) {
	  try {
		if (TaskManagerEvent.BEFORE_EXECUTE.equals(event)) {
		  taskExecutionUI.beforeExecute(task);
		} else if (TaskManagerEvent.AFTER_EXECUTE.equals(event)) {
		  taskExecutionUI.afterExecute(task);
		} else if (TaskManagerEvent.SUBMITTED.equals(event)) {
		  taskExecutionUI.submitted(task);
		}
	  } catch (Exception e) {
		Application.getHandlerExceptionManager().handle(
			new TaskManagerExeception(e));
	  }
	}
  }

}
