package org.japura.task.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.SwingUtilities;
import javax.swing.Timer;

import org.japura.task.Task;
import org.japura.util.InputEventBlocker;

/**
 * <P>
 * Copyright (C) 2012 Carlos Eduardo Leite de Andrade
 * <P>
 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * <P>
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * <P>
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <A
 * HREF="www.gnu.org/licenses/">www.gnu.org/licenses/</A>
 * <P>
 * For more information, contact: <A HREF="www.japura.org">www.japura.org</A>
 * <P>
 * 
 * @author Carlos Eduardo Leite de Andrade
 */
public abstract class AbstractTaskExecutionUI implements TaskExecutionUI{

  private int delay = 500;
  private int count;
  private Starter starter;
  private InputEventBlocker eventHook = new InputEventBlocker();

  /**
   * Sets a delay for the {@link #delayedStart()} method.
   * 
   * @param delay
   *          the delay
   * 
   * @see #delayedStart()
   */
  public void setDelay(int delay) {
	delay = Math.max(delay, 0);
	this.delay = delay;
  }

  /**
   * Gets the delay for the {@link #delayedStart()} method.
   * 
   * @return the delay
   * 
   * @see #delayedStart()
   */
  public int getDelay() {
	return delay;
  }

  /**
   * Invoked when the first task is submitted.
   */
  protected abstract void initialize();

  /**
   * Invoked in the event dispatch thread after a specified delay whether the
   * tasks are not finished.
   * 
   * @see #getDelay()
   * @see #setDelay(int)
   */
  protected abstract void delayedStart();

  /**
   * Invoked in the event dispatch thread before a task execution.
   */
  protected abstract void update(Task<?> task);

  /**
   * Invoked in the event dispatch thread after tasks executions.
   */
  protected abstract void finish();

  @Override
  public void submitted(Task<?> task) {
	count++;
	if (count == 1) {
	  eventHook.apply();
	  initialize();
	  starter = new Starter();
	  starter.start();
	}
  }

  @Override
  public void beforeExecute(final Task<?> task) {
	SwingUtilities.invokeLater(new Runnable() {
	  @Override
	  public void run() {
		update(task);
	  }
	});
  }

  @Override
  public void afterExecute(Task<?> task) {
	count--;
	if (count == 0) {
	  starter.stop();
	  starter = null;
	  SwingUtilities.invokeLater(new Runnable() {
		@Override
		public void run() {
		  eventHook.remove();
		  finish();
		}
	  });
	}
  }

  private class Starter extends Timer implements ActionListener{

	private static final long serialVersionUID = 1L;
	private boolean stop;

	public Starter() {
	  super(AbstractTaskExecutionUI.this.getDelay(), null);
	  addActionListener(this);
	  setRepeats(false);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
	  if (stop == false) {
		AbstractTaskExecutionUI.this.delayedStart();
		eventHook.remove();
	  }
	}

	@Override
	public void stop() {
	  stop = true;
	  super.stop();
	}
  }

}
